C-*-FORTRAN-*-
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C +++ Luminosities
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      REAL FUNCTION ainv_lumi(IE)
C------------------------------------------------------------------------------
C     Intergrated luminosity (pb-1) of the ALEPH data samples analyzed
C
C     Input:     IE    sqrt(s) index (1=189, 2=192, 3=196, 4=200, 5=202,
C                                     6=205, 7=207)
C------------------------------------------------------------------------------
      IMPLICIT   NONE
      INTEGER    IE,iana
      REAL       LUMI(7)
C - Data statements
      DATA       LUMI /176.3,28.93,79.85,86.27,41.89,999.,999./
C - Statements
      ainv_lumi= 0.
      IF( IE.GE.1 .AND. IE.LE.7 ) ainv_lumi= LUMI(IE)
C - Over !
      RETURN
      END 

      REAL FUNCTION ainv_lumi_y2k(IE)
C------------------------------------------------------------------------------
C     Intergrated luminosity (pb-1) of the ALEPH data samples analyzed
C
C     Special function for Y2K
C
C     Input:     IE    sqrt(s) index (1=202, 2=203, 3=204, 4=205, 5=206,
C                                     6=207, 7=208)
C------------------------------------------------------------------------------
      IMPLICIT   NONE
      INTEGER    IE,iana
      REAL       LUMI(7)
C - Data statements
      DATA       LUMI /0.0,1.76,8.32,71.56,65.52,60.81,7.31/
C - Statements
      ainv_lumi_y2k= 0.
      IF( IE.GE.1 .AND. IE.LE.7 ) ainv_lumi_y2k= LUMI(IE)
C - Over !
      RETURN
      END 

      DOUBLE PRECISION FUNCTION ainv_ecm_y2k(IE)
C------------------------------------------------------------------------------
C     Center of Mass energies of the ALEPH data samples analyzed
C
C     Special function for Y2K
C
C     Input:     IE    sqrt(s) index (1=202, 2=203, 3=204, 4=205, 5=206,
C                                     6=207, 7=208)
C------------------------------------------------------------------------------
      IMPLICIT   NONE
      INTEGER    IE,iana
      REAL*8     ECM(7)
C - Data statements
      DATA       ECM /201.87d0,202.67d0,203.79d0,205.03d0,
     .                206.31d0,206.61d0,207.95d0/
C - Statements
      ainv_ecm_y2k= 0d0
      IF( IE.GE.1 .AND. IE.LE.7 ) ainv_ecm_y2k= ECM(IE)
C - Over !
      RETURN
      END 

C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C +++ Efficiencies
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      REAL FUNCTION ainv_eff(IE,C,MH)
C------------------------------------------------------------------------------
C     Get efficiency
C
C     Input:     IE    sqrt(s) index (1=189, 2=192, 3=196, 4=200, 5=202,
C                                     6=205, 7=207)
C                 C    channel (1=hadronic, 2=leptonic)
C                MH    Invisible Higgs mass 
C------------------------------------------------------------------------------

      IMPLICIT   NONE
      INTEGER    IE, C
      REAL       MH
      REAL       EFF
      REAL effhad189_ainv,effhad192_ainv
      REAL effhad196_ainv,effhad200_ainv,effhad202_ainv
      REAL efflep189_ainv,efflep192_ainv
      REAL efflep196_ainv,efflep200_ainv,efflep202_ainv


C---- Data statements

      ainv_eff= 0.

      if (IE.eq.1) then
         if (C.eq.1) then
            eff = 0.98*effhad189_ainv(MH)
         elseif (C.eq.2) then
            eff = efflep189_ainv(MH)
         endif
      elseif (IE.eq.2) then
         if (C.eq.1) then
            eff = 0.98*effhad192_ainv(MH)
         elseif (C.eq.2) then
            eff = 0.
         endif
      elseif (IE.eq.3) then
         if (C.eq.1) then
            eff = 0.98*effhad196_ainv(MH)
         elseif (C.eq.2) then
            eff = efflep196_ainv(MH)
         endif
      elseif (IE.eq.4) then
         if (C.eq.1) then
            eff = 0.98*effhad200_ainv(MH)
         elseif (C.eq.2) then
            eff = efflep200_ainv(MH)
         endif
      elseif (IE.eq.5) then
         if (C.eq.1) then
            eff = 0.98*effhad202_ainv(MH)
         elseif (C.eq.2) then
            eff = efflep202_ainv(MH)
         endif
      elseif (IE.eq.6) then
         if (C.eq.1) then
            eff = 999.
         elseif (C.eq.2) then
            eff = 999.
         endif
      elseif (IE.eq.7) then
         if (C.eq.1) then
            eff = 999.
         elseif (C.eq.2) then
            eff = 0.
         endif
      endif

C---- Output

      ainv_eff= MAX(EFF,0.)

      RETURN
      END 

C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C +++ Background
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ainv_bkg(IE,C,MH,BKG)
C------------------------------------------------------------------------------
C     Input:     IE    sqrt(s) index (1=189, 2=192, 3=196, 4=200, 5=202,
C                                     6=205, 7=207)
C                 C    channel (1=hadrons, 2=leptons)
C                MH    Higgs mass 
C     Output:    BKG   total bkg
C
C------------------------------------------------------------------------------
      IMPLICIT   NONE

      INTEGER    IE, C
      REAL       MH,BKG
      REAL bachad189_ainv,bachad196_ainv
      real bachad200_ainv,bachad202_ainv
      real bachad205_ainv,bachad207_ainv

      if (IE.eq.1) then
         if (C.eq.1) then
            BKG = bachad189_ainv(MH)
         elseif (C.eq.2) then
            BKG = 4.55
         endif
      elseif (IE.eq.2) then
         if (C.eq.1) then
            BKG = 2.40
         elseif (C.eq.2) then
            BKG = 1.0
         endif
      elseif (IE.eq.3) then
         if (C.eq.1) then
            BKG = bachad196_ainv(MH)
         elseif (C.eq.2) then
            BKG = 2.92
         endif
      elseif (IE.eq.4) then
         if (C.eq.1) then
            BKG = bachad200_ainv(MH)
         elseif (C.eq.2) then
            BKG = 2.99
         endif
      elseif (IE.eq.5) then
         if (C.eq.1) then
            BKG = bachad202_ainv(MH)
         elseif (C.eq.2) then
            BKG = 1.49
         endif
      elseif (IE.eq.6) then
         if (C.eq.1) then
            BKG = bachad205_ainv(MH)
         elseif (C.eq.2) then
            BKG = 6.70
         endif
      elseif (IE.eq.7) then
         if (C.eq.1) then
            BKG = bachad207_ainv(MH)
         elseif (C.eq.2) then
            BKG = 0.
         endif
      endif

      RETURN
      END 


C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C +++ Mass distributions
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      REAL*8 FUNCTION ainv_bkg_dis(M,MH,C,IE)
C------------------------------------------------------------------------------
C     Bkg mass distribution
C
C     Input:     IE    sqrt(s) index (1=189, 2=192, 3=196, 4=200, 5=202,
C                                     6=205, 7=207)
C                 C    channel (1=hadronic, 2=leptonic)
C                 M    reconstructed mass
C                MH    Higgs reference mass (needed because of the sliding cut)
C------------------------------------------------------------------------------ 
      IMPLICIT   NONE
      integer ie,c
      real*8 m,mh
      real*8 background_had_ainv
      real background_lep_ainv,background_lepy2k_ainv
      external background_had_ainv
      external background_lep_ainv
      external background_lepy2k_ainv

      if (IE.eq.1) then
         if (C.eq.1) then
            ainv_bkg_dis = (background_had_ainv(m,mh))
         elseif (C.eq.2) then
            ainv_bkg_dis = dble(background_lep_ainv(sngl(m)))
         endif
      elseif (IE.ge.6.and.C.eq.2) then
         ainv_bkg_dis = dble(background_lepy2k_ainv(sngl(m)))
      else
         ainv_bkg_dis = 1d0
      endif

      END 
      REAL*8 FUNCTION ainv_sig_dis(M,MH,C,IE)
C------------------------------------------------------------------------------
C     Sig mass distribution
C
C     Input:     IE    sqrt(s) index (1=189, 2=192, 3=196, 4=200, 5=202,
C                                     6=205, 7=207)
C                 C    channel (1=hadronic, 2=leptonic)
C                 M    reconstructed mass
C                MH    Higgs reference mass (needed because of the sliding cut)
C------------------------------------------------------------------------------ 
      IMPLICIT   NONE
      integer ie,c,ihz
      real*8 m,mh
      real*8 signal_had_ainv,signal_lep_ainv
      real*8 signal_lep_ainv_204,signal_lep_ainv_205
      real*8 signal_lep_ainv_206,signal_lep_ainv_207
      real*8 signal_lep_ainv_208
      real*4 effy2klep_ainv
      external signal_had_ainv,signal_lep_ainv
      external signal_lep_ainv_204,signal_lep_ainv_205
      external signal_lep_ainv_206,signal_lep_ainv_207
      external signal_lep_ainv_208
      real*8 ainv_ecm_y2k,ainv_hz_y2k
      real*4 ainv_lumi_y2k
      real*4 lumi20,lumi22,lumi23,lumi24,lumi25
      real*4 lumi26,lumi27,lumi28
      real*8 ecm20,ecm22,ecm23,ecm24,ecm25
      real*8 ecm26,ecm27,ecm28
c
      ihz = 0
      lumi23 = ainv_lumi_y2k(2)
      lumi24 = ainv_lumi_y2k(3)
      lumi25 = ainv_lumi_y2k(4)
      lumi26 = ainv_lumi_y2k(5)
      lumi27 = ainv_lumi_y2k(6)
      lumi28 = ainv_lumi_y2k(7)
      ecm23  = ainv_ecm_y2k(2)
      ecm24  = ainv_ecm_y2k(3)
      ecm25  = ainv_ecm_y2k(4)
      ecm26  = ainv_ecm_y2k(5)
      ecm27  = ainv_ecm_y2k(6)
      ecm28  = ainv_ecm_y2k(7)
c
      if (IE.eq.1) then
         if (C.eq.1) then
            ainv_sig_dis = signal_had_ainv(m,mh)
         elseif (C.eq.2) then
            ainv_sig_dis = signal_lep_ainv(m,sngl(mh))
         endif
cJBdV: a 203 GeV, shapes de 204 GeV...
      elseif (IE.eq.6.and.C.eq.2) then
         ainv_sig_dis =      
     .  (ainv_hz_y2k(ecm23,mh,ihz)
     . * dble(lumi23)*signal_lep_ainv_204(m,mh)
     . * dble(effy2klep_ainv(sngl(mh),203.))
     . + ainv_hz_y2k(ecm24,mh,ihz)
     . * dble(lumi24)*signal_lep_ainv_204(m,mh)
     . * dble(effy2klep_ainv(sngl(mh),204.))
     . + ainv_hz_y2k(ecm25,mh,ihz)
     . * dble(lumi25)*signal_lep_ainv_205(m,mh)
     . * dble(effy2klep_ainv(sngl(mh),205.))
     . + ainv_hz_y2k(ecm26,mh,ihz)
     . * dble(lumi26)*signal_lep_ainv_206(m,mh)
     . * dble(effy2klep_ainv(sngl(mh),206.))
     . + ainv_hz_y2k(ecm27,mh,ihz)
     . * dble(lumi27)*signal_lep_ainv_207(m,mh)
     . * dble(effy2klep_ainv(sngl(mh),207.))
     . + ainv_hz_y2k(ecm28,mh,ihz)
     . * dble(lumi28)*signal_lep_ainv_208(m,mh)
     . * dble(effy2klep_ainv(sngl(mh),208.)))
      else
         ainv_sig_dis = 1d0
      endif

      END 

C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C +++ Candidates
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ainv_candidates(IE,C,MH,NC,MR)
C------------------------------------------------------------------------------
C     Get number of candidates acting onto a given channel C,
C     for a given energy IE, for a given mass hypothesis MH;
C
C     Input:     IE    sqrt(s) index (1=189, 2=192, 3=196, 4=200, 5=202,
C                                     6=205, 7=207)
C                 C    channel  (1=hadrons, 2=leptons)
C                MH    Higgs reference mass 
C     Output:    NC    Number of observed candidates
C                MR(1,...,23) Reconstructed masses 
C
C------------------------------------------------------------------------------
      IMPLICIT   NONE
      
      real MH,MR(23),v(23)
      integer IE,C,NC,i
      integer nhad189_ainv,nhad196_ainv
      integer nhad200_ainv,nhad202_ainv     
      integer nhad205_ainv,nhad207_ainv     

      do i = 1,23
         mr(i) = 0.
      enddo

      if (IE.eq.1) then
         if (C.eq.1) then
            call recm189_ainv(MH,v)
            NC = nhad189_ainv(MH)
            do i = 1,NC
               MR(i) = v(i)
            enddo
         elseif (C.eq.2) then
            NC = 5
            MR(1) = 75.70
            MR(2) = 75.82
            MR(3) = 82.10
            MR(4) = 87.14
            MR(5) = 92.61
         endif
      elseif (IE.eq.2) then
         if (C.eq.1) then
            NC = 0
         elseif (C.eq.2) then
            NC = 0
         endif
      elseif (IE.eq.3) then
         if (C.eq.1) then
            NC = nhad196_ainv(MH)
         elseif (C.eq.2) then
            NC = 4
         endif
      elseif (IE.eq.4) then
         if (C.eq.1) then
            NC = nhad200_ainv(MH)
         elseif (C.eq.2) then
            NC = 1
         endif
      elseif (IE.eq.5) then
         if (C.eq.1) then
            NC = nhad202_ainv(MH)
         elseif (C.eq.2) then
            NC = 2
         endif
      elseif (IE.eq.6) then
         if (C.eq.1) then
            NC = nhad205_ainv(MH)
         elseif (C.eq.2) then
            NC = 7
            MR(1) = 78.56  
            MR(2) = 82.18   
            MR(3) = 90.61
            MR(4) = 91.99
            MR(5) = 93.38
            MR(6) = 98.51
            MR(7) = 99.89
         endif
      elseif (IE.eq.7) then
         if (C.eq.1) then
            NC = nhad207_ainv(MH)
         elseif (C.eq.2) then
            NC = 0
         endif
      endif
      end

